
//adapted from http://en.radzio.dxp.pl/sed1520/
//adapted for DIP180 (180x32 pixel with 3 SED1520 controller)
//adapted by Dr. Christian Legl and Stephan Gamp

#include "font.h"
#include <avr/io.h>
#include <util/delay.h>


#define DISPLAY_ON  		0b10101111 //0xAF
#define DISPLAY_OFF 		0b10101110 //0xAE
#define DISPLAY_START_LINE 	0b11000000 //0xC0 RAM Linie 0
#define PAGE_ADDRESS_SET 	0b10111000 //0xB8 RAM Page 0
#define COLUMN_ADDRESS_SET 	0b00000000 //0x00 RAM Column address
#define ADC_CLOCKWISE 		0xA0
#define ADC_COUNTERCLOCKWISE 0xA1
#define STATIC_DRIVE_ON 	0xA5
#define STATIC_DRIVE_OFF 	0xA4
#define DUTY_RATIO_16 		0xA8
#define DUTY_RATIO_32 		0xA9
#define READ_MODIFY_WRITE 	0b11100000 //0xE0 on
#define END_READ_MODIFY     0b11101110 //0xEE off
#define RESET 				0b11100010 //0xE2 SW Reset
#define SCREEN_WIDTH	    180 //for DIP180
#define SCREEN_HEIGHT		32 //for DIP180

#define SED1520_DATA_PORT 	PORTD //Data Port
#define SED1520_DATA_DDR 	DDRD
#define SED1520_DATA_PIN 	PIND

#define SED1520_CONTROL_PORT 	PORTA //Control Port for RW, Ao, E1,E2,E3,RES
#define SED1520_CONTROL_DDR 	DDRA


//--------------------------------------------------------------
/* used ports on the Atmega 16
Atmega Display		Atmega	Display
A0 	= 	n.c. 		D0	=	D0
A1 	= 	E3			D1	=	D1
A2 	= 	E2			D2	=	D2
A3 	= 	RW			D3	=	D3
A4 	= 	A0			D4	=	D4
A5 	= 	E1			D5	=	D5
A6 	= 	RES			D6	=	D6
A7 	= 	n.c.		D7	=	D7
*/
//---------------------------------------------------------------


#define SED1520_A0  (1 << 4)
#define SED1520_E1  (1 << 5)
#define SED1520_E2  (1 << 2)
#define SED1520_E3  (1 << 1)
#define SED1520_RW  (1 << 3)
#define SED1520_RES (1 << 6)

//Requests Controller Status
#define CTRL_BUSY 		0x80 //Controller still working
#define CTRL_ADC_CW 	0x40 //CW output
#define CTRL_DISPLAY_ON 0x20 //Display on
#define CTRL_RESET_ON 	0x10 //Being Reset

extern unsigned char lcd_x, lcd_y;


/*
void GLCD_WaitForStatus(unsigned char,unsigned char);
void GLCD_WriteCommand(unsigned char,unsigned char);
void GLCD_GoTo(unsigned char,unsigned char);
void GLCD_WriteData(unsigned char);
unsigned char GLCD_ReadData(void);
void GLCD_ClearScreen(void);
void GLCD_WriteChar(char);
void GLCD_WriteString(char *);
void GLCD_SetPixel(unsigned char, unsigned char, unsigned char);
void GLCD_Init(void);
void GLCD_Bitmap(char * , unsigned char, unsigned char, unsigned char, unsigned char);
*/

//-------------------------------------------------------------------------------------------------
//
//-------------------------------------------------------------------------------------------------

void GLCD_InitPorts(void)
{

_delay_ms(1000); //waiting for Display Controller being ready

SED1520_DATA_DDR = 0xFF; //Data Port as Ouptut

// all necessary Control Ports as output
SED1520_CONTROL_DDR |= (SED1520_E2 | SED1520_E1 | SED1520_E3 | SED1520_RW | SED1520_A0 | SED1520_RES);


_delay_ms(100);  //initalize for 68-MPU RES Pin H-L-H
SED1520_CONTROL_PORT |= SED1520_RES; 
_delay_ms(100);
SED1520_CONTROL_PORT &= ~SED1520_RES; 
_delay_ms(100);
SED1520_CONTROL_PORT |= SED1520_RES; 
_delay_ms(100);



}
//-------------------------------------------------------------------------------------------------
//
//-------------------------------------------------------------------------------------------------
void GLCD_WaitForStatus(unsigned char status, unsigned char controller)
{
char tmp=1;
SED1520_CONTROL_PORT &= ~SED1520_A0; //A0 -> L, needed for commands
SED1520_CONTROL_PORT |= SED1520_RW;  //RW -> H, read
SED1520_CONTROL_PORT &= ~SED1520_E1; //E1 -> L,	 ensure rising edge afterwards
SED1520_DATA_DDR = 0x00; //Dataport as Input



do
  {
  if(controller == 0) 
    {
	SED1520_CONTROL_PORT |= SED1520_E1; //E1 -> H, reading caused by rising edge of E1
	asm("nop");asm("nop"); //wait
	tmp = SED1520_DATA_PIN; // handover status to tmp
	SED1520_CONTROL_PORT &= ~SED1520_E1; //E1 -> L
	} 
  
  if(controller == 1) 
    {
	SED1520_CONTROL_PORT |= SED1520_E2; 
	asm("nop");asm("nop"); 
	tmp = SED1520_DATA_PIN; 
	SED1520_CONTROL_PORT &= ~SED1520_E2; 
	}
	
	if(controller == 2) 
    {
	SED1520_CONTROL_PORT |= SED1520_E3; 
	asm("nop");asm("nop"); 
	tmp = SED1520_DATA_PIN; 
	SED1520_CONTROL_PORT &= ~SED1520_E3; 
	}
  }while(tmp & status); //check if Controller is finished
SED1520_DATA_DDR = 0xFF;  //Dataport as Output
}
//-------------------------------------------------------------------------------------------------
// Write command
//-------------------------------------------------------------------------------------------------
void GLCD_WriteCommand(unsigned char commandToWrite,unsigned char ctrl)
{
GLCD_WaitForStatus(CTRL_BUSY, ctrl); //waiting for Controller ready

SED1520_CONTROL_PORT &= ~SED1520_A0; //A0 -> L , Commands
SED1520_CONTROL_PORT &= ~SED1520_RW; //RW -> L , write

SED1520_DATA_PORT = commandToWrite; //put command on data port




if(ctrl==0)
  {
  SED1520_CONTROL_PORT |= SED1520_E1; //E1 -> H
  asm("nop");asm("nop"); //Pause
  SED1520_CONTROL_PORT &= ~SED1520_E1; //E1 -> L, takeover on falling edge
  }


if(ctrl==1)
  {
  SED1520_CONTROL_PORT |= SED1520_E2;
  asm("nop");asm("nop");
   SED1520_CONTROL_PORT &= ~SED1520_E2;
  }

  
if(ctrl==2)
  {
  SED1520_CONTROL_PORT |= SED1520_E3;
  asm("nop");asm("nop");
  SED1520_CONTROL_PORT &= ~SED1520_E3;
  }  
  
}
//-------------------------------------------------------------------------------------------------
// Write data
//-------------------------------------------------------------------------------------------------
void GLCD_WriteData(unsigned char dataToWrite)
{
GLCD_WaitForStatus(CTRL_BUSY, 0);
GLCD_WaitForStatus(CTRL_BUSY, 1);
GLCD_WaitForStatus(CTRL_BUSY, 2);
SED1520_CONTROL_PORT |= SED1520_A0;   //A0 -> H, Data
SED1520_CONTROL_PORT &= ~SED1520_RW;  //RW -> L. write
SED1520_DATA_PORT = dataToWrite;      //put Data on Dataport
if(lcd_x < (SCREEN_WIDTH/3)) 
  {
  SED1520_CONTROL_PORT |= SED1520_E1; //E1 -> H
  asm("nop");asm("nop");              //Pause
  SED1520_CONTROL_PORT &= ~SED1520_E1;//E1 -> L, takeover by falling edge
  }

if((lcd_x >= (SCREEN_WIDTH/3)) && (lcd_x < (SCREEN_WIDTH*2/3)))
  {
  SED1520_CONTROL_PORT |= SED1520_E2;
  asm("nop");asm("nop");
  SED1520_CONTROL_PORT &= ~SED1520_E2;
  }
  
if(lcd_x >= (SCREEN_WIDTH*2/3))
  {
  SED1520_CONTROL_PORT |= SED1520_E3;
  asm("nop");asm("nop");
  SED1520_CONTROL_PORT &= ~SED1520_E3;
  }

lcd_x++;
if(lcd_x >= SCREEN_WIDTH)
	lcd_x = 0;
}
//-------------------------------------------------------------------------------------------------
// Read data
//-------------------------------------------------------------------------------------------------
unsigned char GLCD_ReadData(void)
{
unsigned char tmp=1;

GLCD_WaitForStatus(CTRL_BUSY, 0); 
GLCD_WaitForStatus(CTRL_BUSY, 1);
GLCD_WaitForStatus(CTRL_BUSY, 2); 
SED1520_CONTROL_PORT |= SED1520_A0; 
SED1520_CONTROL_PORT |= SED1520_RW; 
SED1520_DATA_DDR = 0x00; //Port as input
SED1520_DATA_PORT = 0xFF;
if(lcd_x <  SCREEN_WIDTH/3)
    {
  	SED1520_CONTROL_PORT |= SED1520_E1;
  	asm("nop");asm("nop");
  	SED1520_CONTROL_PORT &= ~SED1520_E1;
	asm("nop");asm("nop");
	SED1520_CONTROL_PORT |= SED1520_E1; 
	asm("nop");asm("nop");
	tmp = SED1520_DATA_PIN; 
	SED1520_CONTROL_PORT &= ~SED1520_E1; 
	};

if((lcd_x >= (SCREEN_WIDTH/3)) && (lcd_x < (SCREEN_WIDTH*2/3))) 
    {	
    SED1520_CONTROL_PORT |= SED1520_E2;
    asm("nop");asm("nop");
    SED1520_CONTROL_PORT &= ~SED1520_E2;
	asm("nop");asm("nop");
	SED1520_CONTROL_PORT |= SED1520_E2;
	asm("nop");asm("nop");
	tmp = SED1520_DATA_PIN;	
	SED1520_CONTROL_PORT &= ~SED1520_E2; 
	};
	
if(lcd_x >= (SCREEN_WIDTH*2/3))
{	
    SED1520_CONTROL_PORT |= SED1520_E3;
    asm("nop");asm("nop");
    SED1520_CONTROL_PORT &= ~SED1520_E3;
	asm("nop");asm("nop");
	SED1520_CONTROL_PORT |= SED1520_E3;
	asm("nop");asm("nop");
	tmp = SED1520_DATA_PIN;	
	SED1520_CONTROL_PORT &= ~SED1520_E3; 
	};
	
SED1520_DATA_DDR = 0xFF; //port as ouput
lcd_x++; 
if(lcd_x > (SCREEN_WIDTH-1))
  lcd_x = 0;
return tmp;
}
//-------------------------------------------------------------------------------------------------
//
//-------------------------------------------------------------------------------------------------
